
/*  
 * $Id: files.c,v 1.27 2005/04/14 07:43:13 isv Exp $
 *
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2004
 * See License.txt for details
 *
 */

#include "module.h"

char * get_path_by_dentry (struct dentry * dentry, struct vfsmount * mnt)
{
	char * tmp = NULL, *path, *page;

	if ( !(page = MEM_ALLOC(PAGE_SIZE)) )
		return NULL;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))
   	path = d_path (dentry, page, PAGE_SIZE);
#else
   	path = d_path (dentry, mnt, page, PAGE_SIZE);
#endif	
	
	if (dentry->d_inode->i_sb->s_magic!=PROC_SUPER_MAGIC
	&& S_ISREG(dentry->d_inode->i_mode)) {
		tmp = MEM_ALLOC(strlen(path) + 10);
		if (tmp) strcpy (tmp, path);
	}
		
	MEM_FREE (page);
   	return tmp;
}

char * get_exec_path(struct task_struct *task, file_info * file)
{
	struct mm_struct * mm;
	struct vm_area_struct * vma;
	struct dentry *dentry;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,48))	
	struct vfsmount *mnt;
#endif	
	char * path;

	path = NULL;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))

#else
	task_lock(task);
#endif	
	mm = task->mm;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))


#else
	if (mm)
		atomic_inc(&mm->mm_users);
	task_unlock(task);
#endif	
	if (!mm)
		goto out;
	down_read(&mm->mmap_sem);
	vma = mm->mmap;
	while (vma) {
		if ((vma->vm_flags & VM_EXECUTABLE) && 
		    vma->vm_file) {
		    dentry = dget(vma->vm_file->f_dentry);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,3,48))		    
			mnt = mntget(vma->vm_file->f_vfsmnt);
			path=get_path_by_dentry(dentry,mnt);
#else
			path=get_path_by_dentry(dentry,NULL);
#endif		
			file->dev = kdev_t_to_nr(dentry->d_inode->i_rdev);
			file->ino = dentry->d_inode->i_ino;
			
			break;
		}
		vma = vma->vm_next;
	}

	up_read(&mm->mmap_sem);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,48))
	atomic_dec(&mm->mm_users);
#endif	
	
out:
	return path;
}



void get_exec_info(struct task_struct *task,  file_info * file)
{
	struct mm_struct * mm;
	struct vm_area_struct * vma;
	struct dentry *dentry;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))

#else
	task_lock(task);
#endif	
	mm = task->mm;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))


#else
	if (mm)
		atomic_inc(&mm->mm_users);
	task_unlock(task);
#endif	
	if (!mm) return;
	
	down_read(&mm->mmap_sem);
	vma = mm->mmap;
	while (vma) {
		if ((vma->vm_flags & VM_EXECUTABLE) && 
		    vma->vm_file) {
		    dentry = dget(vma->vm_file->f_dentry);
			file->dev = kdev_t_to_nr(dentry->d_inode->i_rdev);
			file->ino = dentry->d_inode->i_ino;
			
			break;
		}
		vma = vma->vm_next;
	}

	up_read(&mm->mmap_sem);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,48))
	atomic_dec(&mm->mm_users);
#endif	
	
}

char * get_file_from_userspace (const char * u_filename, file_info * info, char ** filename)
{
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,3,48))
	struct nameidata nd;
#else
	struct dentry * dentry = NULL;
#endif	
	char * k_path = 0;
	*filename = NULL;
	
	/* Get kernel-space filename if user part is present */
	if ( !u_filename )
		return 0;

	/* 
	 * Fill nameidata structure if the file exists.
	 * If the file isn't exists, there is nothing to test.
	 */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,3,48))
	if ( user_path_walk(u_filename, &nd) )
		return 0;
			
	k_path = get_path_by_dentry (nd.dentry, nd.mnt);
	*filename	= k_path;

	info->dev = kdev_t_to_nr(nd.dentry->d_inode->i_rdev);
	info->ino = nd.dentry->d_inode->i_ino;


	path_release(&nd);
#else
	
	dentry = namei(u_filename);

	if ( IS_ERR(dentry) )
		return 0;

	k_path		= get_path_by_dentry (dentry, NULL);
	*filename	= k_path;
	
	info->dev = kdev_t_to_nr(dentry->d_inode->i_rdev);
	info->ino = dentry->d_inode->i_ino;
	
	dput (dentry);
#endif			

	return k_path;
}
