#!/usr/bin/perl
use strict;
use IPC::Open2;
use File::Basename;

# Really useful when script is run from the relative path
BEGIN { unshift @INC, $1 if $0 =~ /(.*)[\/]/; }
use klinstall;

my ($line, %sections, $sec, $oldconfig, $inifile, $kldbedit, $db_name, $users_db_name, %keeper_sections);

my $newconfig = 'STDOUT';
my $mod_name = "smtpscan";
my $template_path = "/etc/kav/5.5/kav4mailservers/templates/";


sub dir_remove_trailing_slashes {
  my $path = shift;
  for(;;) {
    my $len;
    my $pos;
    $len = length($path);
    if ($len < 2) {
    # Could be '/' or any other character. Ok.
      return $path;
    }
    $pos = rindex($path, '/');
    if ($pos != $len - 1) {
    # No trailing slash
      return $path;
    }
    # Remove the trailing slash
    $path = substr($path, 0, $len - 1)
  }
}

sub internal_dirname {
  my $path = shift;
  my $pos;

  $path = dir_remove_trailing_slashes($path);

  $pos = rindex($path, '/');
  if ($pos == -1) {
    # No slash
    return '.';
  }

  if ($pos == 0) {
    # The only slash is at the beginning
    return '/';
  }

  return substr($path, 0, $pos);
}
	
sub config_dump
{

	open (INI,"< $inifile") or die "Could not open file $inifile: $!";
	while($line=<INI>)
	{
		if ($line=~/^\[(.*)\]$/)
		{
			$sec=$1;
		}
		elsif ($line=~/^(.*)\=(.*)/)
		{
			$sections{$sec}{$1}=$2 ;
		}
	}
	$oldconfig=$sections{'AVP32'}{'DefaultProfile'};	
	unless ($oldconfig=~/^\/.*/)
	{
		my $dir=internal_dirname($inifile);
		$oldconfig=~ s/^(\.\/)(.*)/$2/;
		$oldconfig="$dir/$oldconfig";
	
	}
	open(OLDCONFIG,"$oldconfig") or die "Could not open old configuraton file: $!";
	while($line=<OLDCONFIG>)
	{
		if ($line=~/^\[(.*)\]$/)
		{
			$sec=$1;
		}
		elsif ($line=~/(.*)\=(.*)/)
		{
			my ($key, $val) = ($1, $2);
			$val =~ s/^yes$/yes/i;
			$val =~ s/^no$/no/i;
			$sections{$sec}{$key}=$val;
		}
	}		
	
	close(OLDCONFIG); 	
}


sub change_macro
{
	my( $str) = @_;
	$str=~s/\%KAVANSWER\%/\%VIRUSNAME\%/m;
	$str=~s/\%RECIPIENT\%/\%RECIPIENTS\%/m;
	return $str;
}

sub get_setting
{
	my( $db_name, $name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my $value = "";
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	if ($first_line=~/^[ \t]*?Name: .*? Value: (.*?)( | Subtopics:.*)$/)
	{
		$value=$1;
	}
	else
	{
		print("Could not get value \"$name\" from db \"$db_name\".\n");
	}
	close(INPUT);
	close(OUTPUT);
	return $value;
}


sub get_mlsetting
{
	my( $db_name, $name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my $value = "";
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	if ($first_line=~/^[ \t]*?Name: .*? Value: (.*?)(| | Subtopics:.*)$/)
	{
		$value=$1;
		while($first_line=<OUTPUT>){
			$first_line =~s/Subtopics://;
			$value = $value.$first_line;
		}
	}
	else
	{
		print("Could not get value \"$name\" from db \"$db_name\".\n");
	}
	close(INPUT);
	close(OUTPUT);
	return $value;
}


sub get_bool_setting
{
	my( $db_name, $name) = @_;
	my $value = get_setting($db_name, $name);
	if($value eq "1"){
		$value="yes";
	}else{
		$value="no";
	}
	return $value;
}


sub get_notbool_setting
{
	my( $db_name, $name) = @_;
	my $value = get_setting($db_name, $name);
	if($value eq "1"){
		$value="no";
	}else{
		$value="yes";
	}
	return $value;
}

sub get_action_setting
{
	my( $db_name, $name) = @_;
	my $value = get_setting($db_name, $name);
	if($value eq "2"){
		$value="cured";
	}elsif($value eq "1"){
		$value="remove";
	}else{
		$value="unchanged";
	}
	return $value;
}

sub get_sublist_as_str
{
	my( $db_name, $name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my $value = "";
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	while( $first_line  = <OUTPUT> )
	{
   		if ($first_line=~/^[ \t]*?Name: (.*?) Value: .*?( | Subtopics:.*)$/)
		{
			my $full_name = $1;
			$full_name =~ s/^$name\///;
			if( !$value eq "" ){
				$value="$value, $full_name";
			}
			else{
				$value="$full_name";
			}
		}	
		else
		{
			print("Could not get value \"$name\" from db \"$db_name\".\n");
		}
	}
	close(INPUT);
	close(OUTPUT);
	return $value;
}

sub get_sublist_as_hash
{
	my( $db_name, $name, $rm_name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my %value;
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	while( $first_line  = <OUTPUT> )
	{
   		if ($first_line=~/^[ \t]*?Name: (.*?) Value: .*?( | Subtopics:.*)$/)
		{
			my $full_name = $1;
			if( $rm_name==1 ){
				$full_name =~ s/^$name\///;
			}
			$value{$full_name}="";
		}	
		else
		{
			print("Could not get value \"$name\" from db \"$db_name\".\n");
		}
	}
	close(INPUT);
	close(OUTPUT);
	return %value;
}


	# main()
	die "Usage: $0 <ms|fs> <newconfpath>\n"
		if scalar @ARGV != 2 || $ARGV[0] !~ /^(m|f)s$/;

	my $mailserver = $ARGV[0] eq "ms" ? 1 : 0;

	if ( -x "/opt/AVP/Tools/kldbedit" )
	{
		$inifile = "/opt/AVP/AvpUnix.ini";
		$kldbedit = "/opt/AVP/Tools/kldbedit";
		$db_name = "/opt/AVP/etc/defUnix";
	}
	elsif ( -x "/usr/local/share/AVP/Tools/kldbedit" )
	{
		$inifile = "/usr/local/share/AVP/etc/AvpUnix.ini";
		$kldbedit = "/usr/local/share/AVP/Tools/kldbedit";
		$db_name = "/usr/local/share/AVP/etc/defUnix";
	}
	else
	{
		die "Could not detect installed version 4.0\n";
	}

	config_dump();

	$keeper_sections{"path"}{"TempPath"}=get_setting($db_name, "GateWarden/directory/tmp");

	$keeper_sections{"smtpscan.report"}{"ReportOk"}=get_notbool_setting($db_name, "GateWarden/log/logonlyinfected");
	$keeper_sections{"smtpscan.limits"}{"MaxRecipient"}=get_setting($db_name, "GateWarden/limits/maxrcpt");
	$keeper_sections{"smtpscan.notify"}{"NotifyFromAddress"}=get_setting($db_name, "GateWarden/daemonlink/daemonname");
	$keeper_sections{"smtpscan.notify"}{"NotSendNotifyTo"}=get_sublist_as_str($db_name, "GateWarden/limits/not_send_notify");

	$sections{'Configuration'}{'KeysPath'} = dirname($inifile) if $sections{'Configuration'}{'KeysPath'} eq ".";
	$sections{'Configuration'}{'BasePath'} = dirname($inifile) if $sections{'Configuration'}{'BasePath'} eq ".";

	my %conf = klinstall::load_ini_file ($ARGV[1])
		or die "Could not read file $ARGV[1]: $!";

	my $config50 = \%conf;

	# Store old values
	my $newbasespath = klinstall::get_ini_value ($config50, "path", "BasesPath");
	my $newkeysspath = klinstall::get_ini_value ($config50, "path", "LicensePath");

	klinstall::output ("Copying license files\n");
	klinstall::copy_files ($sections{'Configuration'}{'KeysPath'}, $newkeysspath, "\.key\$|\.dat\$");

	klinstall::output ("Copying anti-virus bases\n");
	klinstall::copy_files ($sections{'Configuration'}{'BasePath'}, $newbasespath, ".+");

	klinstall::output ("Importing the old settings\n");

	klinstall::set_ini_value ($config50, "scanner.options", "ExcludeDirs", $sections{'Configuration'}{'ExcludeDir'});
	klinstall::set_ini_value ($config50, "scanner.options", "ExcludeMask", $sections{'Configuration'}{'ExcludeMask'});
	klinstall::set_ini_value ($config50, "scanner.options", "Packed", $sections{'Object'}{'Packed'});
	klinstall::set_ini_value ($config50, "scanner.options", "Archives", $sections{'Object'}{'Archives'});
	klinstall::set_ini_value ($config50, "scanner.options", "SelfExtArchives", $sections{'Object'}{'SelfExtArchives'});
	klinstall::set_ini_value ($config50, "scanner.options", "MailBases", $sections{'Object'}{'MailBases'});
	klinstall::set_ini_value ($config50, "scanner.options", "MailPlain", $sections{'Object'}{'MailPlain'});
	klinstall::set_ini_value ($config50, "scanner.options", "Heuristic", $sections{'Object'}{'CodeAnalyser'});
	klinstall::set_ini_value ($config50, "scanner.options", "LocalFS", $sections{'Object'}{'CrossFs'} =~ /^y/i ? "no" : "yes" );

	klinstall::set_ini_value ($config50, "scanner.options", "Recursion", $sections{'Object'}{'SubDirectories'});

	klinstall::set_ini_value ($config50, "scanner.report", "Append", $sections{'Report'}{'Append'});
	klinstall::set_ini_value ($config50, "scanner.report", "ShowOk", $sections{'Report'}{'ShowOK'});


	if ( $mailserver )
	{
		klinstall::set_ini_value ($config50, "path", "TempPath", $keeper_sections{'path'}{'TempPath'});
		klinstall::set_ini_value ($config50, "aveserver.report", "Append", $sections{'Report'}{'Append'});
		klinstall::set_ini_value ($config50, "smtpscan.report", "Append", $keeper_sections{'smtpscan.report'}{'ReportOk'});
		klinstall::set_ini_value ($config50, "smtpscan.limits", "MaxRecipient", $keeper_sections{'smtpscan.limits'}{'MaxRecipient'});

		klinstall::set_ini_value ($config50, "smtpscan.notify", "NotifyFromAddress", $keeper_sections{'smtpscan.notify'}{'NotifyFromAddress'});
		klinstall::set_ini_value ($config50, "smtpscan.notify", "NotSendNotifyTo", $keeper_sections{'smtpscan.notify'}{'NotSendNotifyTo'});

		my %groups = get_sublist_as_hash($db_name, "GateWarden/Groups", 1);
		my (%msg_types, %templates);
		$msg_types{"Infected"}="infected";
		$msg_types{"Suspicious"}="suspicouse";
		$msg_types{"Cured"}="cured";
		$msg_types{"Corrupted"}="corrupted";
		$msg_types{"Filtered"}="filtered";
		$msg_types{"Corrupted"}="corrupted";

		foreach my $grp_name (keys (%groups))
		{
			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "Senders", get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/from"))
				if get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/from");

			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "Recipients", get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/to"))
				if get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/to");

			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "Check", get_bool_setting($db_name, "GateWarden/Groups/$grp_name/check"));
			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "FilterByName", get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/filter"));
			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "FilterByMIME", get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/filter_mime"));
			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "FilterNotCheckName", get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/not_check"));
			klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "FilterNotCheckMIME", get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/not_check_mime"));
	
			foreach my $msg_type (keys (%msg_types))
			{
				my $old_msg_type = $msg_types{$msg_type};

				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}Quarantine", get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/isolator"));
				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}AdminNotify", get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/admin_send_notify"));
				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}AdminAction", get_action_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/admin_action"));
				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}SenderNotify", get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/sender_send_notify"));
				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}RecipientNotify", get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/user_send_notify"));
				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}RecipientAttachReport", get_notbool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/user_block_mail"));
				klinstall::set_ini_value ($config50, "smtpscan.group:$grp_name", "${msg_type}RecipientAction", get_action_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/user_action"));
			}
		}

		$keeper_sections{"recipient"}{"Charset"}=get_setting($db_name, "GateWarden/messages/recipient_msg/charset");
		$keeper_sections{"recipient"}{"Subject"}=change_macro(get_setting($db_name, "GateWarden/messages/recipient_msg/subject"));
		$keeper_sections{"recipient"}{"ContentType"}=get_setting($db_name, "GateWarden/messages/recipient_msg/content_type");
		$keeper_sections{"recipient"}{"Content"}=change_macro(get_mlsetting($db_name, "GateWarden/messages/recipient_msg/content"));

		$keeper_sections{"sender"}{"Charset"}=get_setting($db_name, "GateWarden/messages/sender_msg/charset");
		$keeper_sections{"sender"}{"Subject"}=change_macro(get_setting($db_name, "GateWarden/messages/sender_msg/subject"));
		$keeper_sections{"sender"}{"ContentType"}=get_setting($db_name, "GateWarden/messages/sender_msg/content_type");
		$keeper_sections{"sender"}{"Content"}=change_macro(get_mlsetting($db_name, "GateWarden/messages/sender_msg/content"));

		$keeper_sections{"admin"}{"Charset"}=get_setting($db_name, "GateWarden/messages/admin_msg/charset");
		$keeper_sections{"admin"}{"Subject"}=change_macro(get_setting($db_name, "GateWarden/messages/admin_msg/subject"));
		$keeper_sections{"admin"}{"ContentType"}=get_setting($db_name, "GateWarden/messages/admin_msg/content_type");
		$keeper_sections{"admin"}{"Content"}=change_macro(get_mlsetting($db_name, "GateWarden/messages/admin_msg/content"));

		foreach my $rcpt_type ("recipient", "sender", "admin")
		{
			my $template_name = $template_path . $rcpt_type . "_notify_template";
			open T, ">$template_name" or die "Could not write template file: $!";
			print T $keeper_sections{$rcpt_type}{"Content"};
			close T;

			foreach my $notify_type ("infected", "suspicious", "cured", "corrupted", "filtered", "error", "protected")
			{
				klinstall::set_ini_value ($config50, "smtpscan.notify.$rcpt_type.$notify_type", "Charset", $keeper_sections{$rcpt_type}{'Charset'});
				klinstall::set_ini_value ($config50, "smtpscan.notify.$rcpt_type.$notify_type", "Subject", $keeper_sections{$rcpt_type}{'Subject'});
				klinstall::set_ini_value ($config50, "smtpscan.notify.$rcpt_type.$notify_type", "ContentType", $keeper_sections{$rcpt_type}{'ContentType'});
				klinstall::set_ini_value ($config50, "smtpscan.notify.$rcpt_type.$notify_type", "Content", $template_name);
			}
		}
    }

	klinstall::save_ini_file ("$ARGV[1].new", %conf);
	exit 0;
