#!/usr/bin/perl

use strict;

sub copy_file
{
	my ($src, $dest) = @_;

	open FSRC, "<$src" or die "Could not read file $src: $!";
	open FDST, ">$dest" or die "Could not write file $dest: $!";
	
	binmode FSRC;
	binmode FDST;
	my $length;

	while ( ($length = sysread (FSRC, my $data, 8192)) > 0 )
	{
		if ( syswrite (FDST, $data, $length) != $length )
		{
			close FSRC;
			close FDST;
				
			unlink $dest;
			die "Could not write file $dest: $!";
		}
	}

	close FSRC;
	close FDST;

	if ( !defined $length )
	{
		unlink $dest;
		die "Couldn't read file $src: $!";
	}
}

sub make_linux_module {
	my $real_os=`perl config/os_detect.pl`;
	my $os_name=`uname -r`;
	
	$os_name=~s/\n//;

	my $kernel_sources="/lib/modules/" .$os_name ."/build";
	my $answer;
	
	$kernel_sources="/usr/src/linux" if ($real_os=~/ALT/ || $real_os=~/Debian/);

	if (!-d "$kernel_sources"  && -d "/usr/src/kernel-headers-$os_name") {

		$kernel_sources="/usr/src/kernel-headers-$os_name";
	}
	

	if (!-f "$kernel_sources/include/linux/version.h") {
	
		if (!-d $kernel_sources || ! -f $kernel_sources . "/Makefile") {
			print "Cannot find kernel source.\n";
		}
	}

	print ("Enter the linux kernel source path [" . $kernel_sources . "]:\n");
	chomp($answer = <STDIN>);

	$kernel_sources=$answer if (!$answer eq '');


	if (! -d $kernel_sources || !(-f $kernel_sources . "/Makefile" || -f "$kernel_sources/include/linux/version.h")) {
		print "Cannot found kernel source at $kernel_sources, please install a kernel source package.\n";
		exit 1;
	}

	my $ret;
	
	$ret = system('./configure --with-kernelsrc=' . $kernel_sources);

	if ($ret) {

		exit(1);
	}
	system('make clean');
	
	$ret = system('make all');

	if ($ret) {
		print("Kavmon module compilation error\n");
		exit(1);
	}
	
	if (!-d "/lib/modules/".$os_name."/misc") {
		$ret=mkdir("/lib/modules/".$os_name."/misc", 0755);
		if (!$ret) {
			print("Cannot create /lib/modules/".$os_name."/misc\n");
			exit(1);
		}
	}
	
	if (-f "bin/kavmon.o") {
		chown("root","bin/kavmon.o");
		copy_file("bin/kavmon.o", "/lib/modules/".$os_name."/misc/kavmon.o") 
	}

	if (-f "bin/kavmon.ko") {
		chown("root","bin/kavmon.ko");
		copy_file("bin/kavmon.ko", "/lib/modules/".$os_name."/misc/kavmon.ko");
	}

	system("/sbin/depmod -a");

	$ret=system("/sbin/modprobe kavmon");
	
	if ($ret !=0) {
		unlink("/lib/modules/".$os_name."/misc/kavmon.o");
		print("Cannot install the kavmon module. Probably you didn't install the appropriate source code for the kernel version you are using. Correct this problem, and run setup again.\n");
		exit(1);
	}

	system("/sbin/rmmod kavmon");
}

sub make_freebsd_module {

	my $ret;
	my $os=`uname -r`;
	
	system("mkdir -p bin");
	system("./configure");
	system("make clean");
	$ret = system("make");

	if ($ret) {
		print("Kavmon module compilation error\n");
		exit(1);
	}


	if ($os=~/5.*/) {
		copy_file("bin/kavmon.ko","/boot/kernel/kavmon.ko");
	} else	{
		copy_file("bin/kavmon.ko","/modules/kavmon.ko");
	}
	$ret = system("kldload kavmon");
	if ($ret !=0) {
		unlink("/modules/kavmon.ko");
		print("Cannot install the kavmon module\n");
		exit(1);
	}

	system("kldunload kavmon");
}


chdir $1 if $0 =~ /(.*)[\/]/;
if ($< != 0) {
   print "Script started without root privileges\n";
   exit(1);
}
	my $answer;

	print ("Would you like to compile the kavmonitor module [Y]: ");
	chomp($answer = <STDIN>);

	if ($answer eq 'n' || $answer eq 'N') {

		exit(0);
	}
	
	my $gcc=`which gcc 2> /dev/null`;
	my $make=`which make 2>/dev/null`;

	if (length($gcc)<1) {
			print "gcc is not found, you need to install gcc package to build the  kavmonitor module\n";
			exit(1);
		}


	if (length($make)<1) {
			print "make is not found, you need to install make package to build the  kavmonitor module\n";
			exit(1);
		}
	

if ($^O=~/linux/i) {
	exit make_linux_module();
	
} else {
	if ($^O=~/freebsd/i) {
		exit make_freebsd_module();
	} else {

		print $^O." OS is not supported\n";
		exit 1;
	}
}

