/* $Id: monitor_helpers.c,v 1.40 2005/03/09 12:23:01 automake Exp $ */

#ifndef __KERNEL__
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "config.h"
#endif
#include "monitor.h"
#include "monitor_helpers.h"

static monitor_callbacks callbacks;

static void Monitor_header(monitor_header * header,u_short data_size, u_short request);
static monitor_data * Monitor_pack_file_answer(u_short queue_id,unsigned long status, char * viruses);
static void Monitor_send_data(FD_TYPE fd, monitor_data *packet);
static monitor_data * Monitor_pack_sign_answer(u_short queue_id,int status);
static monitor_data * Monitor_pack_msg_answer(u_short queue_id,enum monitor_msg_action action);

static monitor_data * Monitor_pack_sign_request(u_short queue_id,const char * filename);

static monitor_data * Monitor_pack_file_request(uid_t uid,pid_t pid,u_short queue_id,
						const char * filename,enum monitor_file_type file_type,
						int flags);
static monitor_data * Monitor_pack_msg_request(uid_t uid,pid_t pid,u_short queue_id,
						enum monitor_msg_type msg_type,const char * msg);



void Monitor_send_sign_answer(FD_TYPE fd,u_short queue_id,int status)
{

	monitor_data * packet;

	packet = Monitor_pack_sign_answer(queue_id,status);
	Monitor_send_data(fd, packet);
	MEM_FREE(packet);
}

void Monitor_send_msg_answer(FD_TYPE fd,u_short queue_id,enum monitor_msg_action action)
{
	monitor_data * packet;

	packet = Monitor_pack_msg_answer(queue_id,action);
	Monitor_send_data(fd, packet);
	MEM_FREE(packet);
}

/* function to send a result from monitor server to monitor client */
void Monitor_send_file_answer(FD_TYPE fd,u_short queue_id,unsigned long status, char * viruses)
{
	monitor_data * packet;

	packet = Monitor_pack_file_answer(queue_id,status,viruses);
	Monitor_send_data(fd, packet);
	MEM_FREE(packet);
}

#ifdef __KERNEL__
/* function to make the request to check a file to the monitor server */
/* allocates a packet that must be free after using */
/* this function is for using in kernel module */
char * Monitor_make_file_request(uid_t uid,pid_t pid,u_short queue_id,const char * filename,
					enum monitor_file_type file_type,
					int flags, int * packet_len)
{
	monitor_data * packet;

	packet = Monitor_pack_file_request(uid,pid,queue_id,filename,file_type,flags);
	*packet_len = packet->header.data_size + sizeof(packet->header);
	return (char *) packet;
}
char * Monitor_make_sign_request(u_short queue_id,const char * filename,int *packet_len)
{
	monitor_data * packet;

	packet = Monitor_pack_sign_request(queue_id,filename);
	*packet_len = packet->header.data_size + sizeof(packet->header);
	return (char *) packet;
}

char * Monitor_make_msg_request(uid_t uid,pid_t pid,u_short queue_id,
				enum monitor_msg_type msg_type,const char * msg, 
				int * packet_len)
{
	monitor_data * packet;

	packet = Monitor_pack_msg_request(uid,pid,queue_id,msg_type,msg);
	*packet_len = packet->header.data_size + sizeof(packet->header);

	return (char *) packet;
}

#endif

void Monitor_send_sign_request(FD_TYPE fd,u_short queue_id,const char * filename)
{
	monitor_data * packet;

	packet = Monitor_pack_sign_request(queue_id,filename);
	Monitor_send_data(fd,packet);
	MEM_FREE(packet);
}


/* function to send the request to check a file to the monitor server */
void Monitor_send_file_request(FD_TYPE fd,uid_t uid,pid_t pid,u_short queue_id,const char * filename,
					enum monitor_file_type file_type,int flags)
{
	monitor_data * packet;

	DEBUG_MSG("%s: pid=%d uid=%squeue_id=%d filename=%s",__FUNCTION__,
			pid,uid,queue_id,filename);
	packet = Monitor_pack_file_request(uid,pid,queue_id,filename,file_type,flags);
	Monitor_send_data(fd,packet);
	MEM_FREE(packet);
}

void Monitor_shutdown_request(FD_TYPE fd)
{
	monitor_data  shutdown;
	Monitor_header(&shutdown.header,0,MONITOR_SHUTDOWN);
	Monitor_send_data(fd,&shutdown);
}

void Monitor_send_msg_request(FD_TYPE fd, uid_t uid,pid_t pid,u_short queue_id,
				enum monitor_msg_type msg_type,const char * msg)
{
	monitor_data * packet;

	DEBUG_MSG("%s: uid=%d pid=%d msg=%s",__FUNCTION__,uid,pid,msg_type,msg);
	packet = Monitor_pack_msg_request(uid,pid,queue_id,msg_type,msg);
	Monitor_send_data(fd,packet);
	MEM_FREE(packet);
}


#ifndef __KERNEL__
void Monitor_check_file_local(uid_t uid, pid_t pid,u_short queue_id,const char * filename,
	                            enum monitor_file_type file_type,int flags)
{
	if (callbacks.file_check_request)
	callbacks.file_check_request(uid,pid,queue_id,filename,file_type,flags);
}
#endif


monitor_data * Monitor_pack_file_answer(u_short queue_id,unsigned long status, char * viruses)
{
	monitor_file_result_data * file;
	int packet_len;
	
	packet_len = sizeof(file->result)+(viruses?strlen(viruses)+1:0);

	file = (monitor_file_result_data *)MEM_ALLOC(sizeof(*file)+packet_len);

	Monitor_header(&file->header,packet_len,MONITOR_ANSWER_FILE);
	memset((char *)&file->result,0, sizeof(file->result));
	file->result.queue_id  = queue_id;
	file->result.result    = status;
	
	if (viruses) {
	    memcpy(file->result.viruses,viruses,strlen(viruses)+1);
	}
	
	return (monitor_data *)file;
}

void Monitor_send_data(FD_TYPE fd, monitor_data *packet)
{
	int packet_len;

	packet_len = packet->header.data_size + sizeof(packet->header);
	MONITOR_WRITE(fd,packet,packet_len);
}

void Monitor_header(monitor_header * header,u_short data_size, u_short request)
{
	memset(header,0,sizeof(*header));
	header->request    = request;
	header->version   = MONITOR_INTERFACE_VERSION;
	header->data_size = data_size;
}

monitor_data * Monitor_pack_sign_answer(u_short queue_id,int status)
{
	monitor_sign_result_data * file;
	int packet_len;
	
	packet_len = sizeof(file->result);

	file = (monitor_sign_result_data *)MEM_ALLOC(sizeof(*file)+1);
	Monitor_header(&file->header,packet_len,MONITOR_ANSWER_SIGN);
	file->result.queue_id  = queue_id;
	file->result.result    = status;
	
	return (monitor_data *)file;
}

monitor_data * Monitor_pack_msg_answer(u_short queue_id,enum monitor_msg_action action)
{
	monitor_msg_result_data * msg;

	msg = (monitor_msg_result_data *)MEM_ALLOC(sizeof(*msg)+1);
	Monitor_header(&msg->header,sizeof(msg->result),MONITOR_ANSWER_MESSAGE);
	msg->result.queue_id  = queue_id;
	msg->result.action    = action;
	
	return (monitor_data *)msg;
}

monitor_data * Monitor_pack_sign_request(u_short queue_id,const char * filename)
{
	monitor_sign_check_data *file;

	file = (monitor_sign_check_data *) MEM_ALLOC(sizeof(*file)+strlen(filename)+2);
	Monitor_header(&file->header,sizeof(file->file)+strlen(filename)+1,MONITOR_CHECK_SIGN);
	file->file.queue_id = queue_id;
	memcpy((char *)file->file.name,filename,strlen(filename)+1);
	return (monitor_data *) file;
}



monitor_data * Monitor_pack_file_request(uid_t uid,pid_t pid,u_short queue_id,const char * filename,
					enum monitor_file_type file_type,int flags)
{
	monitor_file_data *file;

	file = (monitor_file_data *) MEM_ALLOC(sizeof(*file)+strlen(filename)+2);
	Monitor_header(&file->header,sizeof(file->file)+strlen(filename)+1,MONITOR_CHECK_FILE);
	file->file.header.pid = pid;
	file->file.header.uid = uid;
	file->file.header.flags = flags;
	file->file.header.queue_id = queue_id;
	file->file.header.file_type = file_type;
	file->file.header.name_len = strlen(filename)+1;
	memcpy((char *)file->file.name,filename,strlen(filename)+1);
	return (monitor_data *) file;
}

monitor_data * Monitor_pack_msg_request(uid_t uid,pid_t pid,u_short queue_id,
					enum monitor_msg_type msg_type,const char * msg)
{
	monitor_msg_data *message;

	message = (monitor_msg_data *) MEM_ALLOC(sizeof(*message)+strlen(msg)+2);
	Monitor_header(&message->header,sizeof(message->msg)+strlen(msg)+1,MONITOR_MESSAGE);

	message->msg.pid = pid;
	message->msg.uid = uid;
	message->msg.queue_id = queue_id;
	message->msg.msg_type = msg_type;
	memcpy((char *)message->msg.message,msg,strlen(msg)+1);

	return (monitor_data *) message;
}

void Monitor_callbacks_init(monitor_callbacks *m_callbacks)
{
	callbacks.file_check_request		= m_callbacks->file_check_request;
	callbacks.file_check_answer		= m_callbacks->file_check_answer;
	callbacks.monitor_shutdown		= m_callbacks->monitor_shutdown;
	callbacks.file_check_sign		= m_callbacks->file_check_sign;
	callbacks.file_answer_sign		= m_callbacks->file_answer_sign;
	callbacks.monitor_msg			= m_callbacks->monitor_msg;
	callbacks.monitor_answer_msg		= m_callbacks->monitor_answer_msg;
}
int Monitor_Request_Ready(TASK_INFO FD_TYPE read_fd,FD_TYPE write_fd, void *data)
{
	monitor_data * request;

	request = (monitor_data *) data;


	switch (request->header.request) {
		case MONITOR_CHECK_FILE:
			{
				monitor_file_data *file_data;

				if (request->header.data_size == 0) {
					DEBUG_MSG("%s:no data found in the packet",__FUNCTION__);
					break;
				}

				file_data = (monitor_file_data *) data;
				switch (file_data->file.header.file_type)
				{
					
					case MONITOR_OPEN_FILE:
					case MONITOR_CLOSE_FILE:
					case MONITOR_EXEC_FILE:
					case MONITOR_UNLINK_FILE:
					case MONITOR_RENAME_FILE:
										
					if (!callbacks.file_check_request) {
						DEBUG_MSG("%s: no file_check_request callback "
						 	  "function found",__FUNCTION__);
						break;
					}

					DEBUG_MSG("%s: package has recieved: "
						   "pid=%d uid=%d queue_id = %d, file=%s flags=%d",
							__FUNCTION__,
							file_data->file.header.pid,
							file_data->file.header.uid,
							file_data->file.header.queue_id,
							file_data->file.name,file_data->file.header.flags);
					
					callbacks.file_check_request(TASK file_data->file.header.uid,
							file_data->file.header.pid,
							file_data->file.header.queue_id,
							file_data->file.name,
							file_data->file.header.file_type,
							file_data->file.header.flags);
					break;
					default:
					DEBUG_MSG("%s: unknown file type=%d",__FUNCTION__,
							file_data->file.header.file_type);
					
				}

			}
		break;
		case MONITOR_ANSWER_FILE:
			{
				monitor_file_result_data * file_data;

				if (!callbacks.file_check_answer) {
					DEBUG_MSG("%s: no file_check_answer callback found",__FUNCTION__);
					break;
				}

				file_data = (monitor_file_result_data *) request;
				DEBUG_MSG("%s: queue_id=%d status=%d",__FUNCTION__,
						file_data->result.queue_id,
						file_data->result.result);

				if (file_data->header.data_size>sizeof(monitor_file_result))
				callbacks.file_check_answer(TASK
						file_data->result.queue_id,
						file_data->result.result,file_data->result.viruses);
				else
				callbacks.file_check_answer(TASK
					file_data->result.queue_id,
					file_data->result.result,NULL);
					
			}

		break;
		case MONITOR_SHUTDOWN:
			if (callbacks.monitor_shutdown) {
				callbacks.monitor_shutdown(TASK_);
				return 1;
			}
			break;

		case MONITOR_CHECK_SIGN:
			{
				monitor_sign_check_data * sign_check;
				sign_check = (monitor_sign_check_data * ) data;

				if (callbacks.file_check_sign) {
					callbacks.file_check_sign(TASK sign_check->file.queue_id,
								sign_check->file.name);
				}

			}

			break;
		case MONITOR_ANSWER_SIGN:
			{
				monitor_sign_result_data * sign_result;
				sign_result = (monitor_sign_result_data * ) data;

				if (callbacks.file_answer_sign) {
					callbacks.file_answer_sign(TASK sign_result->result.queue_id,
								sign_result->result.result);
				}
			}
			break;

		case MONITOR_MESSAGE:
			{
				monitor_msg_data * msg_data;
				
				msg_data = (monitor_msg_data *) data;
				if (callbacks.monitor_msg) {
					callbacks.monitor_msg(TASK msg_data->msg.uid,
								msg_data->msg.pid,
								msg_data->msg.queue_id,
								msg_data->msg.msg_type,
								msg_data->msg.message);
				}
			}
		break;
		case MONITOR_ANSWER_MESSAGE:
			{
				monitor_msg_result_data * msg_data;
				
				msg_data = (monitor_msg_result_data *) data;
				if (callbacks.monitor_answer_msg) {
					callbacks.monitor_answer_msg(TASK msg_data->result.queue_id,
									msg_data->result.action);
				}
			}
		break;
		
		
		default:
		DEBUG_MSG("%s:unknown request from the monitor module =%d",__FUNCTION__,
					request->header.request);

		}

	return 0;
}


