
/*  
 * $Id: module.c,v 1.45 2005/03/31 16:35:03 isv Exp $
 *
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2004
 * See License.txt for details
 *
 */

#include "module.h"
#include <linux/module.h>
#include <linux/init.h>


/* Module section */
MODULE_AUTHOR ("Kaspersky Lab");
MODULE_DESCRIPTION ("Kaspersky Anti-Virus Monitor Module for Linux.");

/* 
 * Global system call table.
 * It is not defined, because it will be imported by the kernel.
 */

void **sys_calls_table;

#ifndef HIDDEN_SCT
extern void *sys_call_table[];
#endif

static atomic_t monitor_count;

void Monitor_ins_use(void)
{
#ifdef LINUX_KERNEL26
	try_module_get(THIS_MODULE);
#else	
        MOD_INC_USE_COUNT;
#endif	

	atomic_inc(&monitor_count);
}

void Monitor_dec_use(void)
{
#ifdef LINUX_KERNEL26
	module_put(THIS_MODULE);
#else	
        MOD_DEC_USE_COUNT;
#endif	

	atomic_dec(&monitor_count);
}

int Monitor_busy(void)
{
	return atomic_read(&monitor_count)?1:0;
}


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))
int init_module(void)
#else
static int __init kavmon_init_module(void)
#endif
	
{
	
	lock_kernel();
#ifdef HIDDEN_SCT
	sys_calls_table = get_sys_call_table_addr();
#else
	sys_calls_table = sys_call_table;
#endif	

	unlock_kernel();	
	
	if (!sys_calls_table) {
	    printk("System calls table can't be found\n");
	    return -EPERM;
	}

	atomic_set(&monitor_count,0);
	
	Monitor_interface_init();
	return 0;
}

#if (LINUX_VERSION_CODE  < KERNEL_VERSION(2,3,48))
void cleanup_module(void)
#else
static void __exit kavmon_cleanup_module(void)
#endif	
{
	Monitor_interface_cleanup();
}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,48))
module_init(kavmon_init_module);
module_exit(kavmon_cleanup_module);
#ifdef MODULE_LICENSE
MODULE_LICENSE ("GPL");
#endif
#endif
