/* $Id: monitor_helpers.h,v 1.40 2005/02/16 14:12:53 automake Exp $ */

#ifndef __MONITOR_HELPERS__
#define __MONITOR_HELPERS__

enum monitor_file_type {
	MONITOR_OPEN_FILE,
	MONITOR_CLOSE_FILE,
	MONITOR_EXEC_FILE,
	MONITOR_UNLINK_FILE,
	MONITOR_RENAME_FILE
};

enum monitor_msg_type {
	MONITOR_ALERT_EXEC_NETWORK,
	MONITOR_ALERT_EXEC_STACK
};


enum monitor_msg_action {
	MONITOR_DENY_ACT,
	MONITOR_TERM_ACT,
	MONITOR_KILL_ACT,
	MONITOR_ABORT_ACT,
	MONITOR_INFO_ACT
};

#ifndef __KERNEL__
static inline enum monitor_file_type convert_to_filetype(int file_type)
{
	enum monitor_file_type type;
	
	
	switch(file_type) {
				case MONITOR_OPEN_FILE:  type = MONITOR_OPEN_FILE; break;
				case MONITOR_CLOSE_FILE: type = MONITOR_CLOSE_FILE; break;
				case MONITOR_EXEC_FILE:  type = MONITOR_EXEC_FILE; break; 
				case MONITOR_UNLINK_FILE:type = MONITOR_UNLINK_FILE; break; 
				case MONITOR_RENAME_FILE:type = MONITOR_RENAME_FILE; break; 
			}
	return type;
}

static inline enum monitor_msg_type convert_to_msgtype(int msg_type)
{
	enum monitor_msg_type type;

	type = MONITOR_ALERT_EXEC_NETWORK;

	switch(msg_type) {
		case MONITOR_ALERT_EXEC_NETWORK: type = MONITOR_ALERT_EXEC_NETWORK; break;
		case MONITOR_ALERT_EXEC_STACK: type = MONITOR_ALERT_EXEC_STACK; break;
	}

	return type;
}

#endif

#define MONITOR_DEBUG "<monitor_debug>"
#ifndef __KERNEL__
#ifdef DEBUG
#define DEBUG_MSG(format, args...) add_log(MONITOR_DEBUG format, ## args)
#else
#define DEBUG_MSG(format, args...)
#endif

#define FD_TYPE int
#define MONITOR_WRITE(fd,packet,len)    write(fd,packet,len)
#define MEM_ALLOC(len)			malloc(len)
#define MEM_FREE(ptr)			free(ptr)
#else
#define FD_TYPE char *

#ifdef __linux__
#include "module.h"
#define MONITOR_WRITE(out_buf,packet,len)
#ifdef DEBUG
#define DEBUG_MSG(format, args...) printk(MONITOR_DEBUG format"\n" , ## args)
#else
#define DEBUG_MSG(format, args...) 
#endif
#endif

#ifdef __FreeBSD__
#include "module.h"
#define MONITOR_WRITE(out_buf,packet,len)
#ifdef DEBUG
#define DEBUG_MSG(format, args...) uprintf(MONITOR_DEBUG format"\n" , ## args)
#else
#define DEBUG_MSG(format, args...) 
#endif
#endif

#endif


typedef struct {
	u_short		archive;
	u_short		packed;
	u_int		viruses;
	u_int		warnings;
	u_int		suspicious;
	u_int		cured;
} kav_stat;



enum monitor_request_type {
	MONITOR_CHECK_SIGN,
	MONITOR_ANSWER_SIGN,
	MONITOR_CHECK_FILE,
	MONITOR_ANSWER_FILE,
	MONITOR_MESSAGE,
	MONITOR_ANSWER_MESSAGE,
	MONITOR_SHUTDOWN

};


/* monitor packet header */ 
typedef struct {
	u_short			request;
	u_short			version;
	u_int			data_size;	

} monitor_header;

/* raw monitor packet */

typedef struct {
	monitor_header		header;
	char			data[0];
} monitor_data;


/* The packets descriptions to send information from the monitor module
 * to the monitor daemon
 */

/* packet header to check files */
typedef struct {
	u_int			uid;
	u_int			pid;
	int			flags;
	u_short			file_type;
	u_short			queue_id;
	u_int			name_len;
} monitor_file_header;

/* data packet to check files */
typedef struct {
	monitor_file_header	header;
	char			name[0];

} monitor_file;

/* raw packet to check files */
typedef struct {
	monitor_header		  header;
	monitor_file		  file;

} monitor_file_data;


/* packet with file status */
typedef struct {
	u_short queue_id;
	u_short notning;
	u_int	result;
	char    viruses[0];
} monitor_file_result;

/* raw packet with file status information */
typedef struct {
	monitor_header		header;
	monitor_file_result	result;

} monitor_file_result_data;

typedef struct {
	u_short			queue_id;
	u_short			nothing;
	char			name[0];
} monitor_sign_check;

typedef struct {
	monitor_header		header;
	monitor_sign_check	file;
} monitor_sign_check_data;


typedef struct {
	u_short			queue_id;
	u_short			result;
} monitor_sign_result;

typedef struct {
	monitor_header		header;
	monitor_sign_result	result;
} monitor_sign_result_data;

typedef struct {
	u_int			uid;
	u_int			pid;
	u_short			queue_id;
	u_short			msg_type;
	char			message[0];
} monitor_msg;

typedef struct {
	monitor_header		header;
	monitor_msg		msg;
} monitor_msg_data;

typedef struct {
	u_short			queue_id;
	u_short			action;
} monitor_msg_result;

typedef struct {
	monitor_header		header;
	monitor_msg_result	result;
} monitor_msg_result_data;

#ifndef __KERNEL__
#define TASK_INFO
#define TASK
#define TASK_
#else
#define TASK_ tsk
#define TASK TASK_,
#endif


typedef void file_check_sign_callback(TASK_INFO u_short ,const char *);
typedef void file_answer_sign_callback(TASK_INFO u_short, u_short);

/* callback function proto is called on the MONITOR_FILE_CHECK request */
typedef void file_check_request_callback(TASK_INFO uid_t, pid_t,u_short, 
					 const char *, enum monitor_file_type,unsigned long);

/* callback function proto is called on the MONITOR_FILE_ANSWER request*/
typedef void file_check_answer_callback (TASK_INFO u_short,unsigned long,char * viruses);

typedef void file_check_inform_callback(TASK_INFO uid_t, pid_t, u_short, const char *,
					enum monitor_file_type,	unsigned long, const char *,
					const kav_stat *);

#ifndef __KERNEL__
typedef void monitor_shutdown_callback(void);
#else
typedef void monitor_shutdown_callback(TASK_INFO_);
#endif

typedef void monitor_msg_callback(TASK_INFO uid_t, pid_t, u_short,enum monitor_msg_type, const char *);
typedef void monitor_msg_answer_callback(TASK_INFO u_short,enum monitor_msg_action action);


typedef struct {
	file_check_request_callback          *file_check_request;
	file_check_answer_callback           *file_check_answer;
	monitor_shutdown_callback            *monitor_shutdown;
	file_check_sign_callback             *file_check_sign;
	file_answer_sign_callback            *file_answer_sign;
	monitor_msg_callback  	             *monitor_msg;
	monitor_msg_answer_callback          *monitor_answer_msg;
	
} monitor_callbacks;

typedef struct {
	file_check_request_callback * file_check_request;
	monitor_shutdown_callback    * server_shutdown;
	file_check_sign_callback     * file_check_sign;
	monitor_msg_callback         * monitor_msg;
	monitor_msg_answer_callback  * monitor_answer_msg;	
} server_params;

#ifdef __cplusplus
extern "C" {
#endif	

#ifndef __KERNEL__	
void monitor_module_config(module_config * config);
/* Initialize monitor in the case of the monitor client appears as a kernel module */
void Monitor_Remote_Init(int remote_client_fd);
int Monitor_Run(void);

/* Initialize the monitor server*/
void Monitor_Server_Init(server_params * params);
void Monitor_client_answer(u_short queue_id,enum monitor_filestat_type status);

void Monitor_answer_sign(u_short queue_id,int result);
void Monitor_answer_msg(u_short queue_id,enum monitor_msg_action action);
void Monitor_Client_Shutdown(void);
#endif
	
void Monitor_send_sign_answer(FD_TYPE fd,u_short queue_id,int status); 

/* send answer on recieved message with the assigned action */
void Monitor_send_msg_answer(FD_TYPE fd,u_short queue_id,enum monitor_msg_action action);


/* send an answer with file status */
void Monitor_send_file_answer(FD_TYPE fd,u_short queue_id,unsigned long status, char * viruses); 

/* send a request to check a file */
void Monitor_send_file_request(FD_TYPE fd, uid_t uid, pid_t pid,u_short queue_id, 
				const char * filename, 
				enum monitor_file_type file_type,int flags);


void Monitor_send_sign_request(FD_TYPE fd,u_short queue_id,const char * filename);

void Monitor_send_msg_request(FD_TYPE fd, uid_t uid,pid_t pid,u_short queue_id,
				enum monitor_msg_type msg_type,const char * msg);

#ifdef __KERNEL__
char * Monitor_make_file_request(uid_t uid,pid_t pid,u_short queue_id,const char * filename,
					enum monitor_file_type file_type,
					int flags, int * packet_len);
char * Monitor_make_sign_request(u_short queue_id,const char * filename, int * packet_len);
char * Monitor_make_msg_request(uid_t uid,pid_t pid,u_short queue_id,enum monitor_msg_type msg_type,
				const char * msg,int * packet_len);
#endif

void Monitor_shutdown_request(FD_TYPE fd);

#ifndef __KERNEL__
void Monitor_check_file_local(uid_t uid, pid_t pid,u_short queue_id,const char * filename,
	                            enum monitor_file_type file_type,int flags);
#endif

/* function to init the callbacks */
void Monitor_callbacks_init(monitor_callbacks *m_callbacks);

/* Handler to parse an arrived request and call the callback functions*/
int Monitor_Request_Ready(TASK_INFO FD_TYPE read_fd,FD_TYPE write_fd, void *data);
#ifdef __cplusplus
}
#endif	
#endif
