
/*  
 * $Id: interface.c,v 1.33 2005/05/20 15:26:39 isv Exp $
 *
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2004
 * See License.txt for details
 *
 */


#include "module.h"

/* monitor proc functions */
static int	monitor_proc_open  (struct inode * inode, struct file * filp );
static ssize_t	monitor_proc_read  (struct file * file, char * ubuf, size_t len, loff_t * offset );


static struct file_operations kavmon_proc_operations = 
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,27))
	owner:		THIS_MODULE,
#endif	
	open:		monitor_proc_open,
	read:		monitor_proc_read
};

static char * monitor_info;
int monitor_major, exploit_major;


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))
static struct inode_operations kavmon_inode_operations = 
{
	&kavmon_proc_operations
};
#endif

int Monitor_interface_init(void)
{
	struct proc_dir_entry *proc_entry;


	Exploit_module_init();
	Monitor_module_init();
	Monitor_queue_init();
	
	/* register the avir monitor device */
	monitor_major = register_chrdev(0, "kavmonitor", &kavmonitor_file_operations);
	register_chrdev(monitor_major, "kavmonitor", &kavmonitor_file_operations); 

	/* register the exploit device */
	exploit_major = register_chrdev(0, "kavexploit", &kavexploit_file_operations);
	register_chrdev(exploit_major, "kavexploit", &kavexploit_file_operations); 

	monitor_info = MEM_ALLOC(PAGE_SIZE);

	proc_entry = create_proc_entry(KAVMON_PROC, 0644, &proc_root);

	if (! proc_entry ) 
		return -ENOENT;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,48))
	proc_entry->proc_fops = &kavmon_proc_operations;
#else
	/* 2.2.x etc */
	proc_entry->ops = &kavmon_inode_operations;
#endif

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,27))
	proc_entry->owner = THIS_MODULE;
#endif

	return 0;
};

int Monitor_interface_cleanup(void)
{
	remove_proc_entry (KAVMON_PROC, &proc_root);
	unregister_chrdev (monitor_major, "kavmonitor");
	unregister_chrdev (exploit_major, "kavexploit");
	MEM_FREE(monitor_info);
	return 0;
}



int monitor_proc_open  (struct inode * inode, struct file * filp)
{
	sprintf(monitor_info,"%d \n%d %d %d\n%d %d %d\n",
			MONITOR_INTERFACE_VERSION,
			monitor_major,0,monitor_started,
			exploit_major,0,exploit_started);

	
	return 0;
}

/*	version
 *	(kavmonitor) major connected(1/0)
 *	(kavexploit) major connected(1/0)
 *
 */

ssize_t	monitor_proc_read  (struct file * file, char * ubuf, size_t len, loff_t * offset)
{
	int count, info_len;

	info_len = strlen(monitor_info);
	
	if (*offset > info_len) return 0;
	
	if (len < info_len - file->f_pos) count = len;
		else count = info_len - file->f_pos;

	copy_to_user(ubuf,monitor_info+file->f_pos,count);
	file->f_pos += count;
	
	*offset+=count;
	
	return count;
}



