
/*  
 * $Id: interface_exploit.c,v 1.13 2005/02/16 14:26:06 isv Exp $
 *
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2004
 * See License.txt for details
 *
 */


#include "module.h"

static int	exploit_open(struct inode * inode, struct file * filp);
static ssize_t	exploit_read(struct file * file, char * ubuf, size_t len, loff_t * offset );
static ssize_t	exploit_write(struct file * file, const char * ubuf, size_t len, loff_t * offset );
static int	exploit_close(struct inode * inode, struct file * filp );
static int      exploit_ioctl (struct inode * inode, struct file *filp, unsigned int cmd,
				 unsigned long addr);
static unsigned int exploit_poll(struct file *file, poll_table * wait );

struct file_operations kavexploit_file_operations = 
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,27))
	owner:		THIS_MODULE,
#endif			
	open:		exploit_open,
	ioctl:		exploit_ioctl,
	release:	exploit_close,
	poll:		exploit_poll,
	read:		exploit_read,
	write:		exploit_write
};

int		exploit_started;		/* set if exploit protection is started */

static int	exploit_daemon_pid;
static int	syscalls_table_altered;

int Exploit_module_init(void)
{
	exploit_daemon_pid = 0;
	syscalls_table_altered = 0;
	exploit_started = 0;
	
	return 0;
};

int Exploit_module_stop(void)
{
	if (!exploit_daemon_pid) return 0;
	
	if (current->pid!=exploit_daemon_pid) return 0;

	if (!monitor_started) {
		if (Monitor_restore_calls()) syscalls_table_altered = 1;
	}
	
	Monitor_queue_cleanup(QUEUE_EXPLOIT);
	exploit_started = 0;

        return 0;
}

int exploit_open (struct inode * inode, struct file * filp)
{
	if (!exploit_daemon_pid) {

	if (syscalls_table_altered) return -EFAULT;
		exploit_daemon_pid = current->pid;
		exploit_started = 1;
						     
		if (!monitor_started) {
			
    	                Monitor_kernel_init();		
			Monitor_intercept_calls();
		}

		Monitor_ins_use();
	}

	return 0;
}

ssize_t exploit_read (struct file * file, char * ubuf, size_t count, loff_t * offset)
{
	if (current->pid!=exploit_daemon_pid) return 0;

	return Exploit_queue_read_interface (current, file, ubuf, count);
}


ssize_t exploit_write (struct file * file, const char * ubuf, size_t count, loff_t * offset)
{
	if (current->pid!=exploit_daemon_pid) return 0;

	return Exploit_queue_write_interface (current, file, ubuf, count, offset);
}


int exploit_close (struct inode * inode, struct file * filp)
{
	if (current->pid!=exploit_daemon_pid) {
		
		return 0;
	}

	if (exploit_started) Exploit_module_stop();
	if (!syscalls_table_altered)  Monitor_dec_use();
	
	exploit_daemon_pid = 0;
	return 0;
}



int exploit_ioctl (struct inode * inode, struct file *filp, unsigned int cmd, unsigned long addr)
{
	switch (cmd) {
		case  EXPLOIT_CONFIG_CMD:
			{
				exploit_config_ioctl * params;
				int i;

				if (m_config.exec_network_prot || m_config.exec_stack_prot) break;

				params = (exploit_config_ioctl *) addr;
				if (!params) return -EFAULT;

				if (get_user(m_config.exec_network_prot,
							&params->exec_network_prot))
					return -EFAULT;

				if (get_user(m_config.exec_stack_prot,
							&params->exec_stack_prot))
					return -EFAULT;
			
				if (m_config.exec_network_prot) {
					char * p;
					char * tmp;

					tmp = MEM_ALLOC(PAGE_SIZE);

					for(i=0;;i++) {
						if (get_user(p,params->exec_net_excludes+i)) break;
						
						if (!p) break;
						if (copy_from_user(tmp,p,strlen_user(p))) {
							MEM_FREE(tmp);
							return -EFAULT;
						}

						Monitor_kernel_add_exec_net_exclude(tmp);
					}
					MEM_FREE(tmp);
				}
				
				if (m_config.exec_stack_prot) {
					char * p;
					char * tmp;

					tmp = MEM_ALLOC(PAGE_SIZE);
										
					for(i=0;;i++) {
						if (get_user(p,params->exec_stack_excludes+i)) break;
						
						if (!p) break;
						if (copy_from_user(tmp,p,strlen_user(p))) {
							MEM_FREE(tmp);
							return -EFAULT;
						}
						Monitor_kernel_add_exec_stack_exclude(tmp);
					}
					MEM_FREE(tmp);
				}
				
			}


			break;
	}

	return -EINVAL; 	
}

unsigned int exploit_poll ( struct file *file, poll_table * wait )
{
	return Exploit_queue_poll_interface (current, file, wait);
}


