
/*  
 * $Id: queue.h,v 1.23 2005/02/16 14:26:05 isv Exp $
 *
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2004
 * See License.txt for details
 *
 */


#ifndef KAVMON_QUEUE_H
#define KAVMON_QUEUE_H

typedef struct {
	int	len;
	int	left;
	char *  data;
} mqueue_data_t;


enum queue_entry_types {
	QUEUE_MONITOR,
	QUEUE_EXPLOIT
};

typedef struct
{
	int				refcnt;
	lock_declare(lock);
	u_short				queue_id;
	int				flags;
	pid_t 				pid;
	uid_t				uid;
	dev_t				devc;
	mqueue_data_t	              * request;
	long				answer;
	enum queue_entry_types		type;
	wait_queue_head_t		waitq;
	struct list_head		list;
} mqueue_entry_t;


#define O_MQUEUE_READY			0x02
#define O_MQUEUE_TESTING		0x04


int  Monitor_queue_init(void);
int Monitor_queue_get_id(void);
void Monitor_queue_set_answer(u_short queue_id, long answer);

void Monitor_queue_wakeup(u_short queue_id);
void Monitor_queue_cleanup(enum queue_entry_types queue_type);
int Monitor_queue_add(enum queue_entry_types queue_type, 
			uid_t uid,pid_t pid, char * request_data, 
			int request_len, long * answer, u_short queue_id);


/* interface functions */
#if defined(__linux__)
/* monitor prototypes */
ssize_t Monitor_queue_read_interface (TASK_INFO struct file * file, char * ubuf, size_t count);
ssize_t Monitor_queue_write_interface (TASK_INFO struct file * file, const char * ubuf, size_t count, loff_t * offset);
unsigned int Monitor_queue_poll_interface (TASK_INFO struct file *file, poll_table * wait);

/* exploits protector prototypes */
ssize_t Exploit_queue_read_interface (TASK_INFO struct file * file, char * ubuf, size_t count);
ssize_t Exploit_queue_write_interface (TASK_INFO struct file * file, const char * ubuf, size_t count, loff_t * offset);
unsigned int Exploit_queue_poll_interface (TASK_INFO struct file *file, poll_table * wait);

#elif defined(__FreeBSD__)
ssize_t Monitor_queue_read_interface (TASK_INFO dev_t dev, struct uio * uio, int ioflags);
ssize_t Monitor_queue_write_interface (TASK_INFO dev_t dev,struct uio *uio, int ioflag);
unsigned int Monitor_queue_poll_interface(TASK_INFO int events);
unsigned int Exploit_queue_poll_interface(TASK_INFO int events);
ssize_t Exploit_queue_read_interface (TASK_INFO dev_t dev, struct uio * uio, int ioflags);
ssize_t Exploit_queue_write_interface (TASK_INFO dev_t dev, struct uio *uio, int ioflag);
#endif
#endif

