
/*  
 * $Id: module.h,v 1.53 2005/04/01 11:11:17 isv Exp $
 *
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2004
 * See License.txt for details
 *
 */

#ifndef KAVMON_MODULE_H
#define KAVMON_MODULE_H

#include <linux/config.h>
#include <linux/version.h>

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,3,48) && LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0) && defined(CONFIG_MODVERSIONS))
#define MODVERSIONS
#include <linux/modversions.h>
#endif


#ifdef UNDEF_SMP
#undef __SMP__
#undef CONFIG_SMP
#else
#if defined(CONFIG_SMP)
#define __SMP__
#endif
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,3,48))
#include <linux/module.h>
#endif
#include <asm/page.h>
#include <linux/vmalloc.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/file.h>
#include <linux/proc_fs.h>
#include <linux/smp_lock.h>
#include <asm/unistd.h>
#include <asm/uaccess.h>
#include <linux/list.h>
#ifdef LINUX_KERNEL26
#include <linux/mount.h>
#include <linux/namei.h>
#include <linux/ptrace.h>


#define kdev_t_to_nr(dev) (dev);

#ifdef HAVE_SYSCALLS_H
#include <linux/syscalls.h>
#endif
#endif


/* Linux 2.2 compatibility  */

#ifndef BUG
 #define BUG() do { \
        printk("kernel BUG at %s:%d!\n", __FILE__, __LINE__); \
	* ((char *) 0) = 0; \
 } while (0)
#endif /* BUG */

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,2,20))
typedef struct wait_queue *wait_queue_head_t;
#define init_waitqueue_head(wait) init_waitqueue(wait)
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))
#define down_read(sem) down(sem)
#define up_read(sem) up(sem)
#define KSTK_EIP(tsk)   (((unsigned long *)(4096+(unsigned long)(tsk)))[1019])
#endif

/* End of compat */


#define MEM_ALLOC(size)		vmalloc(size)
#define MEM_FREE(ptr)		vfree(ptr)



static inline pid_t get_pgrp(struct task_struct * tsk)
{
#ifdef LINUX_KERNEL26
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,5))
	return tsk->signal->pgrp;
#else	
	return process_group(tsk); 
#endif

#else
	return tsk->pgrp; 
#endif	

}


#ifndef for_each_process
#define for_each_process(p) for_each_task(p)
#endif

#ifndef MIN
#define MIN(a, b) ( (a) < (b) ? (a) : (b) )
#endif

#define PRN(format, args...) printk(format"\n" , ## args)

#define lock_declare(lock)	rwlock_t lock

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,48))
#define lock_init(lock,name) do { } while(0)
#define strlen_user(str) strnlen_user(str, ~0UL >> 1)
#else
#define lock_init(lock,name)	rwlock_init(lock)
#endif

void Monitor_ins_use(void);
void Monitor_dec_use(void);
int Monitor_busy(void);


#define TASK_INFO_ struct task_struct * tsk
#define TASK_INFO TASK_INFO_, 

#include "kavmon.h"
#include "md5.h"
#include "monitor.h"
#include "monitor_helpers.h"
#include "queue.h"
#include "util.h"
#include "kernel.h"
#include "cache.h"
#include "interface.h"
#include "interface_monitor.h"
#include "interface_exploit.h"
#include "interceptor.h"
#include "files.h"
#include "utils.h"


#endif /* KAVHDR_MODULE_H */
